@@ -10,9 +10,14 @@
 #undef HAVE_MDB
 #undef HAVE_ORACLE
 #undef HAVE_PRIMEBASE
-#undef HAVE_TDS
+#undef HAVE_FREETDS
+#undef HAVE_FREETDS_VER0_5X
+#undef HAVE_FREETDS_VER0_6X
 #undef HAVE_SYBASE
+#undef HAVE_IBMDB2
 #undef SYBASE_DEBUG
 #undef HAVE_POSTGRES
 #undef HAVE_CAMEL
+#undef HAVE_SQLITE
 #undef GETTEXT_PACKAGE
+#undef BONOBO_EXPLICIT_TRANSLATION_DOMAIN
