@@ -21,8 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include <stdlib.h>
 #include <string.h>
+#include <config.h>
 #include <glib.h>
 #include <gmodule.h>
 #include <libgda/gda-config.h>
@@ -31,10 +32,11 @@
 #include <libgda/gda-log.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
+#include <sys/stat.h>
 
-#ifndef LIBGDA_USER_CONFIG_FILE
-#define LIBGDA_USER_CONFIG_FILE "/.libgda/config"  /* Appended to $HOME */
-#endif
+#define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
+#define LIBGDA_USER_CONFIG_FILE LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S \
+				"config"
 
 //FIXME: many functions are not thread safe!
 
@@ -42,8 +44,8 @@ typedef struct {
     gchar *name;
     gchar *type;
     gchar *value;
-    gchar *mtime;
-    gchar *muser;
+    //gchar *mtime;
+    //gchar *muser;
 } gda_config_entry;
 
 typedef struct {
@@ -76,25 +78,36 @@ gda_config_read_entries (xmlNodePtr cur)
 		if (!strcmp(cur->name, "entry")){
 			entry = g_new (gda_config_entry, 1);
 			entry->name = xmlGetProp(cur, "name");
-			if (entry->name == NULL)
+			if (entry->name == NULL){
 				g_warning ("NULL 'name' in an entry");
+				entry->name = g_strdup ("");
+			}
 
 			entry->type =  xmlGetProp(cur, "type");
-			if (entry->type == NULL)
+			if (entry->type == NULL){
 				g_warning ("NULL 'type' in an entry");
+				entry->type = g_strdup ("");
+			}
 
 			entry->value =  xmlGetProp(cur, "value");
-			if (entry->value == NULL)
+			if (entry->value == NULL){
 				g_warning ("NULL 'value' in an entry");
+				entry->value = g_strdup ("");
+			}
 
+			/*
 			entry->muser =  xmlGetProp(cur, "muser");
-			if (entry->value == NULL)
+			if (entry->value == NULL){
 				g_warning ("NULL 'muser' in an entry");
-
+				entry->muser = g_strdup ("");
+			}
+			
 			entry->mtime =  xmlGetProp(cur, "mtime");
-			if (entry->value == NULL)
+			if (entry->value == NULL){
 				g_warning ("NULL 'mtime' in an entry");
-
+				entry->mtime = g_strdup ("");
+			}
+			*/
 			list = g_list_append (list, entry);
 		} else {
 			g_warning ("'entry' expected, got '%s'. Ignoring...", 
@@ -174,15 +187,37 @@ get_config_client ()
 		xmlKeepBlanksDefault(0);
 		if (g_file_get_contents (LIBGDA_GLOBAL_CONFIG_FILE, &full_file,
 					 &len, NULL)){
-			config_client->global = 
-				gda_config_parse_config_file (full_file, len);
+			config_client->global = gda_config_parse_config_file (full_file, len);
+			g_free (full_file);
+
 		}
 
 		user_config = g_strdup_printf ("%s%s", g_get_home_dir (),
 						LIBGDA_USER_CONFIG_FILE);
 		if (g_file_get_contents (user_config, &full_file, &len, NULL)){
-			config_client->user =
-				gda_config_parse_config_file (full_file, len);
+			config_client->user = gda_config_parse_config_file (full_file, len);
+			g_free (full_file);
+		} else {
+			if (!g_file_test (user_config, G_FILE_TEST_EXISTS)){
+				gchar *dirpath;
+				FILE *fp;
+
+				dirpath = g_strdup_printf ("%s%s", g_get_home_dir (),
+								LIBGDA_USER_CONFIG_DIR);
+				if (!g_file_test (dirpath, G_FILE_TEST_IS_DIR)){
+					if (mkdir (dirpath, 0700))
+						g_warning ("Error creating directory %s", dirpath);
+				}
+
+				fp = fopen (user_config, "wt");
+				if (fp == NULL)
+					g_warning ("Unable to create the configuration file.");
+				else
+					fclose (fp);
+
+				g_free (dirpath);
+			} else
+				g_warning ("Config file is not readable.");
 		}
 		g_free (user_config);
 	}
@@ -315,11 +350,13 @@ add_xml_entry (xmlNodePtr parent, gda_config_entry *entry)
 	xmlNodePtr new_node;
 
 	new_node = xmlNewTextChild (parent, NULL, "entry", NULL);
-	xmlSetProp (new_node, "name", entry->name ? entry->name : "");
-	xmlSetProp (new_node, "type", entry->type ? entry->type : "");
-	xmlSetProp (new_node, "value", entry->value ? entry->value : "");
-	xmlSetProp (new_node, "mtime", entry->name ? entry->name : "");
-	xmlSetProp (new_node, "muser", entry->type ? entry->type : "");
+	xmlSetProp (new_node, "name", entry->name);
+	xmlSetProp (new_node, "type", entry->type);
+	xmlSetProp (new_node, "value", entry->value);
+	/*
+	xmlSetProp (new_node, "mtime", entry->mtime);
+	xmlSetProp (new_node, "muser", entry->muser);
+	*/
 }
 
 static xmlNodePtr
@@ -346,8 +383,6 @@ write_config_file ()
 	gchar *user_config;
 
 	cfg_client = get_config_client ();
-	g_return_if_fail (cfg_client->user != NULL);
-
 	doc = xmlNewDoc ("1.0");
 	g_return_if_fail (doc != NULL);
 	root = xmlNewDocNode (doc, NULL, "libgda-config", NULL);
@@ -754,6 +789,9 @@ gda_config_has_section (const gchar *path)
 
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, path);
+	if (section == NULL)
+		section = gda_config_search_section (cfg_client->global, path);
+
 	return (section != NULL) ? TRUE : FALSE;
 }
 
@@ -775,9 +813,44 @@ gda_config_has_key (const gchar *path)
 
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, NULL);
+	if (entry == NULL)
+		entry = gda_config_search_entry (cfg_client->global,
+						 path,
+						 NULL);
+
 	return (entry != NULL) ? TRUE : FALSE;
 }
 
+/**
+ * gda_config_get_type
+ * @path: path to the configuration key
+ *
+ * Gets a string representing the type of the value of the given key.
+ * The caller is responsible of freeing the returned value.
+ *
+ * Returns: NULL if not found. Otherwise: "string", "float", "long", "bool".
+ */
+gchar *
+gda_config_get_type (const gchar *path)
+{
+	gda_config_entry *entry;
+	gda_config_client *cfg_client;
+
+	g_return_val_if_fail (path != NULL, FALSE);
+
+	cfg_client = get_config_client ();
+	entry = gda_config_search_entry (cfg_client->user, path, NULL);
+	if (entry == NULL)
+		entry = gda_config_search_entry (cfg_client->global, 
+						 path,
+						 NULL);
+
+	if (entry == NULL)
+		return NULL;
+
+	return g_strdup (entry->type);
+}
+
 /**
  * gda_config_list_sections
  * @path: path for root dir
@@ -937,7 +1010,7 @@ gda_config_get_provider_list (void)
 		ext = g_strrstr (name, ".");
 		if (!ext)
 			continue;
-		if (strcmp (ext, ".so"))
+		if (strcmp (ext + 1, G_MODULE_SUFFIX))
 			continue;
 
 		path = g_build_path (G_DIR_SEPARATOR_S, LIBGDA_PLUGINDIR,
@@ -1087,6 +1160,11 @@ gda_config_get_data_source_list (void)
 		info->username = gda_config_get_string (tmp);
 		g_free (tmp);
 
+		/* get the password */
+		tmp = g_strdup_printf ("%s/%s/Password", GDA_CONFIG_SECTION_DATASOURCES, (char *) l->data);
+		info->password = gda_config_get_string (tmp);
+		g_free (tmp);
+
 		list = g_list_append (list, info);
 	}
 
@@ -1138,6 +1216,7 @@ gda_config_copy_data_source_info (GdaDataSourceInfo *src)
 	info->cnc_string = g_strdup (src->cnc_string);
 	info->description = g_strdup (src->description);
 	info->username = g_strdup (src->username);
+	info->password = g_strdup (src->password);
 
 	return info;
 }
@@ -1155,6 +1234,7 @@ gda_config_free_data_source_info (GdaDataSourceInfo *info)
 	g_free (info->cnc_string);
 	g_free (info->description);
 	g_free (info->username);
+	g_free (info->password);
 
 	g_free (info);
 }
@@ -1185,12 +1265,13 @@ gda_config_get_data_source_model (void)
 	GList *l;
 	GdaDataModel *model;
 
-	model = gda_data_model_array_new (5);
+	model = gda_data_model_array_new (6);
 	gda_data_model_set_column_title (model, 0, _("Name"));
 	gda_data_model_set_column_title (model, 1, _("Provider"));
 	gda_data_model_set_column_title (model, 2, _("Connection string"));
 	gda_data_model_set_column_title (model, 3, _("Description"));
 	gda_data_model_set_column_title (model, 4, _("Username"));
+	gda_data_model_set_column_title (model, 5, _("Password"));
 	
 	/* convert the returned GList into a GdaDataModelArray */
 	dsn_list = gda_config_get_data_source_list ();
@@ -1205,6 +1286,7 @@ gda_config_get_data_source_model (void)
 		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->cnc_string));
 		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->description));
 		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->username));
+		value_list = g_list_append (value_list, gda_value_new_string ("******"));
 
 		gda_data_model_append_row (GDA_DATA_MODEL (model), value_list);
 	}
@@ -1222,6 +1304,7 @@ gda_config_get_data_source_model (void)
  * @cnc_string: Connection string for the new data source.
  * @description: Description for the new data source.
  * @username: User name for the new data source.
+ * @password: Password to use when authenticating @username.
  *
  * Adds a new data source (or update an existing one) to the GDA
  * configuration, based on the parameters given.
@@ -1231,38 +1314,52 @@ gda_config_save_data_source (const gchar *name,
 			     const gchar *provider,
 			     const gchar *cnc_string,
 			     const gchar *description,
-			     const gchar *username)
+			     const gchar *username,
+			     const gchar *password)
 {
-	gchar *tmp;
+	GString *str;
+	gint trunc_len;
 
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (provider != NULL);
 
+	str = g_string_new ("");
+	g_string_printf (str, "%s/%s/", GDA_CONFIG_SECTION_DATASOURCES, name);
+	trunc_len = strlen (str->str);
+	
 	/* set the provider */
-	tmp = g_strdup_printf ("%s/%s/Provider", GDA_CONFIG_SECTION_DATASOURCES, name);
-	gda_config_set_string (tmp, provider);
-	g_free (tmp);
+	g_string_append (str, "Provider");
+	gda_config_set_string (str->str, provider);
+	g_string_truncate (str, trunc_len);
 
 	/* set the connection string */
 	if (cnc_string) {
-		tmp = g_strdup_printf ("%s/%s/DSN", GDA_CONFIG_SECTION_DATASOURCES, name);
-		gda_config_set_string (tmp, cnc_string);
-		g_free (tmp);
+		g_string_append (str, "DSN");
+		gda_config_set_string (str->str, cnc_string);
+		g_string_truncate (str, trunc_len);
 	}
 
 	/* set the description */
 	if (description) {
-		tmp = g_strdup_printf ("%s/%s/Description", GDA_CONFIG_SECTION_DATASOURCES, name);
-		gda_config_set_string (tmp, description);
-		g_free (tmp);
+		g_string_append (str, "Description");
+		gda_config_set_string (str->str, description);
+		g_string_truncate (str, trunc_len);
 	}
 
 	/* set the username */
-	if (cnc_string) {
-		tmp = g_strdup_printf ("%s/%s/Username", GDA_CONFIG_SECTION_DATASOURCES, name);
-		gda_config_set_string (tmp, username);
-		g_free (tmp);
+	if (username) {
+		g_string_append (str, "Username");
+		gda_config_set_string (str->str, username);
+		g_string_truncate (str, trunc_len);
 	}
+
+	/* set the password */
+	if (password) {
+		g_string_append (str, "Password");
+		gda_config_set_string (str->str, password);
+		g_string_truncate (str, trunc_len);
+	}
+	g_string_free (str, TRUE);
 }
 
 /**
