@@ -24,7 +24,7 @@
 #if !defined(__libgda_h__)
 #  define __libgda_h__
 
-//#include <libgda/gda-batch.h>
+/*#include <libgda/gda-batch.h> */
 #include <libgda/gda-client.h>
 #include <libgda/gda-command.h>
 #include <libgda/gda-config.h>
@@ -42,6 +42,7 @@
 #include <libgda/gda-row.h>
 #include <libgda/gda-select.h>
 #include <libgda/gda-table.h>
+#include <libgda/gda-transaction.h>
 #include <libgda/gda-util.h>
 #include <libgda/gda-value.h>
 #include <libgda/gda-xml-connection.h>
