@@ -26,6 +26,7 @@
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gda-mysql.h"
 #include "gda-mysql-recordset.h"
 
@@ -38,6 +39,7 @@ static void gda_mysql_provider_init       (GdaMysqlProvider *provider,
 					   GdaMysqlProviderClass *klass);
 static void gda_mysql_provider_finalize   (GObject *object);
 
+static const gchar *gda_mysql_provider_get_version (GdaServerProvider *provider);
 static gboolean gda_mysql_provider_open_connection (GdaServerProvider *provider,
 						    GdaConnection *cnc,
 						    GdaQuarkList *params,
@@ -45,6 +47,8 @@ static gboolean gda_mysql_provider_open_connection (GdaServerProvider *provider,
 						    const gchar *password);
 static gboolean gda_mysql_provider_close_connection (GdaServerProvider *provider,
 						     GdaConnection *cnc);
+static const gchar *gda_mysql_provider_get_server_version (GdaServerProvider *provider,
+							   GdaConnection *cnc);
 static const gchar *gda_mysql_provider_get_database (GdaServerProvider *provider,
 						     GdaConnection *cnc);
 static gboolean gda_mysql_provider_change_database (GdaServerProvider *provider,
@@ -92,8 +96,10 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_mysql_provider_finalize;
+	provider_class->get_version = gda_mysql_provider_get_version;
 	provider_class->open_connection = gda_mysql_provider_open_connection;
 	provider_class->close_connection = gda_mysql_provider_close_connection;
+	provider_class->get_server_version = gda_mysql_provider_get_server_version;
 	provider_class->get_database = gda_mysql_provider_get_database;
 	provider_class->change_database = gda_mysql_provider_change_database;
 	provider_class->create_database = gda_mysql_provider_create_database;
@@ -153,6 +159,17 @@ gda_mysql_provider_new (void)
 	return GDA_SERVER_PROVIDER (provider);
 }
 
+/* get_version handler for the GdaMysqlProvider class */
+static const gchar *
+gda_mysql_provider_get_version (GdaServerProvider *provider)
+{
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
+
+	return VERSION;
+}
+
 /* open_connection handler for the GdaMysqlProvider class */
 static gboolean
 gda_mysql_provider_open_connection (GdaServerProvider *provider,
@@ -258,6 +275,25 @@ gda_mysql_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	return TRUE;
 }
 
+/* get_server_version handler for the GdaMysqlProvider class */
+static const gchar *
+gda_mysql_provider_get_server_version (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	MYSQL *mysql;
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	
+	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
+	if (!mysql) {
+		gda_connection_add_error_string (cnc, _("Invalid MySQL handle"));
+		return FALSE;
+	}
+
+	return (const gchar *) mysql->server_version;
+}
+
 static GList *
 process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 {
@@ -360,6 +396,7 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 				    const gchar *name)
 {
 	gint rc;
+	gchar *sql;
 	MYSQL *mysql;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
 
@@ -373,7 +410,9 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	rc = mysql_create_db (mysql, name);
+	sql = g_strdup_printf ("CREATE DATABASE %s", name);
+	rc = mysql_query (mysql, sql);
+	g_free (sql);
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
@@ -389,6 +428,7 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 				  const gchar *name)
 {
 	gint rc;
+	gchar *sql;
 	MYSQL *mysql;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
 
@@ -402,7 +442,9 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	rc = mysql_drop_db (mysql, name);
+	sql = g_strdup_printf ("DROP DATABASE %s", name);
+	rc = mysql_query (mysql, sql);
+	g_free (sql);
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
@@ -460,6 +502,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
@@ -467,6 +510,30 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	/* set isolation level */
+	switch (gda_transaction_get_isolation_level (xaction)) {
+	case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ COMMITTED", 46);
+		break;
+	case GDA_TRANSACTION_ISOLATION_READ_UNCOMMITTED :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED", 48);
+		break;
+	case GDA_TRANSACTION_ISOLATION_REPEATABLE_READ :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL REPEATABLE READ", 47);
+		break;
+	case GDA_TRANSACTION_ISOLATION_SERIALIZABLE :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE", 44);
+		break;
+	default :
+		rc = 0;
+	}
+
+	if (rc != 0) {
+		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		return FALSE;
+	}
+
+	/* start the transaction */
 	rc = mysql_real_query (mysql, "BEGIN", strlen ("BEGIN"));
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
@@ -554,21 +621,32 @@ gda_mysql_provider_supports (GdaServerProvider *provider,
 }
 
 static void
-add_string_row (GdaDataModelArray *recset, const gchar *str)
+add_aggregate_row (GdaDataModelArray *recset, const gchar *str, const gchar *comments)
 {
 	GdaValue *value;
-	GList list;
+	GList *list;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
 
-	value = gda_value_new_string (str);
-	list.data = value;
-	list.next = NULL;
-	list.prev = NULL;
+	/* 1st the name */
+	list = g_list_append (NULL, gda_value_new_string (str));
+	/* 2nd the unique id */
+	list = g_list_append (list, gda_value_new_string (str));
+	/* 3rd the owner */
+	list = g_list_append (list, gda_value_new_null ());
+	/* 4th the comments */ 
+	list = g_list_append (list, gda_value_new_string (comments));
+	/* 5th the return type */
+	list = g_list_append (list, gda_value_new_type (GDA_VALUE_TYPE_UNKNOWN));
+	/* 6th the argument type */
+	list = g_list_append (list, gda_value_new_type (GDA_VALUE_TYPE_UNKNOWN));
+	/* 7th the SQL definition */
+	list = g_list_append (list, gda_value_new_null ());
 
-	gda_data_model_append_row (GDA_DATA_MODEL (recset), &list);
+	gda_data_model_append_row (GDA_DATA_MODEL (recset), list);
 
-	gda_value_free (value);
+	g_list_foreach (list, gda_value_free, NULL);
+	g_list_free (list);
 }
 
 static GdaDataModel *
@@ -586,142 +664,142 @@ get_mysql_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
 
 	/* fill the recordset */
-	add_string_row (recset, "abs");
-	add_string_row (recset, "acos");
-	add_string_row (recset, "adddate");
-	add_string_row (recset, "ascii");
-	add_string_row (recset, "asin");
-	add_string_row (recset, "atan");
-	add_string_row (recset, "atan2");
-	add_string_row (recset, "benchmark");
-	add_string_row (recset, "bin");
-	add_string_row (recset, "bit_count");
-	add_string_row (recset, "ceiling");
-	add_string_row (recset, "char");
-	add_string_row (recset, "char_length");
-	add_string_row (recset, "character_length");
-	add_string_row (recset, "coalesce");
-	add_string_row (recset, "concat");
-	add_string_row (recset, "concat_ws");
-	add_string_row (recset, "connection_id");
-	add_string_row (recset, "conv");
-	add_string_row (recset, "cos");
-	add_string_row (recset, "cot");
-	add_string_row (recset, "count");
-	add_string_row (recset, "curdate");
-	add_string_row (recset, "current_date");
-	add_string_row (recset, "current_time");
-	add_string_row (recset, "current_timestamp");
-	add_string_row (recset, "curtime");
-	add_string_row (recset, "database");
-	add_string_row (recset, "date_add");
-	add_string_row (recset, "date_format");
-	add_string_row (recset, "date_sub");
-	add_string_row (recset, "dayname");
-	add_string_row (recset, "dayofmonth");
-	add_string_row (recset, "dayofweek");
-	add_string_row (recset, "dayofyear");
-	add_string_row (recset, "decode");
-	add_string_row (recset, "degrees");
-	add_string_row (recset, "elt");
-	add_string_row (recset, "encode");
-	add_string_row (recset, "encrypt");
-	add_string_row (recset, "exp");
-	add_string_row (recset, "export_set");
-	add_string_row (recset, "extract");
-	add_string_row (recset, "field");
-	add_string_row (recset, "find_in_set");
-	add_string_row (recset, "floor");
-	add_string_row (recset, "format");
-	add_string_row (recset, "from_days");
-	add_string_row (recset, "from_unixtime");
-	add_string_row (recset, "get_lock");
-	add_string_row (recset, "greatest");
-	add_string_row (recset, "hex");
-	add_string_row (recset, "hour");
-	add_string_row (recset, "if");
-	add_string_row (recset, "ifnull");
-	add_string_row (recset, "inet_aton");
-	add_string_row (recset, "inet_ntoa");
-	add_string_row (recset, "insert");
-	add_string_row (recset, "instr");
-	add_string_row (recset, "interval");
-	add_string_row (recset, "isnull");
-	add_string_row (recset, "last_insert_id");
-	add_string_row (recset, "lcase");
-	add_string_row (recset, "least");
-	add_string_row (recset, "left");
-	add_string_row (recset, "length");
-	add_string_row (recset, "load_file");
-	add_string_row (recset, "locate");
-	add_string_row (recset, "log");
-	add_string_row (recset, "log10");
-	add_string_row (recset, "lower");
-	add_string_row (recset, "lpad");
-	add_string_row (recset, "ltrim");
-	add_string_row (recset, "make_set");
-	add_string_row (recset, "master_pos_wait");
-	add_string_row (recset, "match");
-	add_string_row (recset, "max");
-	add_string_row (recset, "md5");
-	add_string_row (recset, "mid");
-	add_string_row (recset, "min");
-	add_string_row (recset, "minute");
-	add_string_row (recset, "mod");
-	add_string_row (recset, "month");
-	add_string_row (recset, "monthname");
-	add_string_row (recset, "now");
-	add_string_row (recset, "nullif");
-	add_string_row (recset, "oct");
-	add_string_row (recset, "octet_length");
-	add_string_row (recset, "ord");
-	add_string_row (recset, "password");
-	add_string_row (recset, "period_add");
-	add_string_row (recset, "period_diff");
-	add_string_row (recset, "pi");
-	add_string_row (recset, "position");	
-	add_string_row (recset, "pow");
-	add_string_row (recset, "power");
-	add_string_row (recset, "quarter");
-	add_string_row (recset, "radians");
-	add_string_row (recset, "rand");
-	add_string_row (recset, "release_lock");
-	add_string_row (recset, "repeat");
-	add_string_row (recset, "replace");	
-	add_string_row (recset, "reverse");
-	add_string_row (recset, "right");
-	add_string_row (recset, "round");
-	add_string_row (recset, "rpad");
-	add_string_row (recset, "rtrim");
-	add_string_row (recset, "second");
-	add_string_row (recset, "sec_to_time");
-	add_string_row (recset, "session_user");
-	add_string_row (recset, "sign");
-	add_string_row (recset, "sin");
-	add_string_row (recset, "soundex");
-	add_string_row (recset, "space");
-	add_string_row (recset, "sqrt");
-	add_string_row (recset, "strcmp");
-	add_string_row (recset, "subdate");
-	add_string_row (recset, "substring");
-	add_string_row (recset, "substring_index");
-	add_string_row (recset, "sysdate");
-	add_string_row (recset, "system_user");
-	add_string_row (recset, "tan");
-	add_string_row (recset, "time_format");
-	add_string_row (recset, "time_to_sec");
-	add_string_row (recset, "to_days");
-	add_string_row (recset, "trim");
-	add_string_row (recset, "truncate");
-	add_string_row (recset, "ucase");
-	add_string_row (recset, "unix_timestamp");
-	add_string_row (recset, "upper");
-	add_string_row (recset, "user");
-	add_string_row (recset, "version");
-	add_string_row (recset, "week");
-	add_string_row (recset, "weekday");
-	add_string_row (recset, "year");
-	add_string_row (recset, "yearweek");
+	add_aggregate_row (recset, "abs", "comments");
+	add_aggregate_row (recset, "acos", "comments");
+	add_aggregate_row (recset, "adddate", "comments");
+	add_aggregate_row (recset, "ascii", "comments");
+	add_aggregate_row (recset, "asin", "comments");
+	add_aggregate_row (recset, "atan", "comments");
+	add_aggregate_row (recset, "atan2", "comments");
+	add_aggregate_row (recset, "benchmark", "comments");
+	add_aggregate_row (recset, "bin", "comments");
+	add_aggregate_row (recset, "bit_count", "comments");
+	add_aggregate_row (recset, "ceiling", "comments");
+	add_aggregate_row (recset, "char", "comments");
+	add_aggregate_row (recset, "char_length", "comments");
+	add_aggregate_row (recset, "character_length", "comments");
+	add_aggregate_row (recset, "coalesce", "comments");
+	add_aggregate_row (recset, "concat", "comments");
+	add_aggregate_row (recset, "concat_ws", "comments");
+	add_aggregate_row (recset, "connection_id", "comments");
+	add_aggregate_row (recset, "conv", "comments");
+	add_aggregate_row (recset, "cos", "comments");
+	add_aggregate_row (recset, "cot", "comments");
+	add_aggregate_row (recset, "count", "comments");
+	add_aggregate_row (recset, "curdate", "comments");
+	add_aggregate_row (recset, "current_date", "comments");
+	add_aggregate_row (recset, "current_time", "comments");
+	add_aggregate_row (recset, "current_timestamp", "comments");
+	add_aggregate_row (recset, "curtime", "comments");
+	add_aggregate_row (recset, "database", "comments");
+	add_aggregate_row (recset, "date_add", "comments");
+	add_aggregate_row (recset, "date_format", "comments");
+	add_aggregate_row (recset, "date_sub", "comments");
+	add_aggregate_row (recset, "dayname", "comments");
+	add_aggregate_row (recset, "dayofmonth", "comments");
+	add_aggregate_row (recset, "dayofweek", "comments");
+	add_aggregate_row (recset, "dayofyear", "comments");
+	add_aggregate_row (recset, "decode", "comments");
+	add_aggregate_row (recset, "degrees", "comments");
+	add_aggregate_row (recset, "elt", "comments");
+	add_aggregate_row (recset, "encode", "comments");
+	add_aggregate_row (recset, "encrypt", "comments");
+	add_aggregate_row (recset, "exp", "comments");
+	add_aggregate_row (recset, "export_set", "comments");
+	add_aggregate_row (recset, "extract", "comments");
+	add_aggregate_row (recset, "field", "comments");
+	add_aggregate_row (recset, "find_in_set", "comments");
+	add_aggregate_row (recset, "floor", "comments");
+	add_aggregate_row (recset, "format", "comments");
+	add_aggregate_row (recset, "from_days", "comments");
+	add_aggregate_row (recset, "from_unixtime", "comments");
+	add_aggregate_row (recset, "get_lock", "comments");
+	add_aggregate_row (recset, "greatest", "comments");
+	add_aggregate_row (recset, "hex", "comments");
+	add_aggregate_row (recset, "hour", "comments");
+	add_aggregate_row (recset, "if", "comments");
+	add_aggregate_row (recset, "ifnull", "comments");
+	add_aggregate_row (recset, "inet_aton", "comments");
+	add_aggregate_row (recset, "inet_ntoa", "comments");
+	add_aggregate_row (recset, "insert", "comments");
+	add_aggregate_row (recset, "instr", "comments");
+	add_aggregate_row (recset, "interval", "comments");
+	add_aggregate_row (recset, "isnull", "comments");
+	add_aggregate_row (recset, "last_insert_id", "comments");
+	add_aggregate_row (recset, "lcase", "comments");
+	add_aggregate_row (recset, "least", "comments");
+	add_aggregate_row (recset, "left", "comments");
+	add_aggregate_row (recset, "length", "comments");
+	add_aggregate_row (recset, "load_file", "comments");
+	add_aggregate_row (recset, "locate", "comments");
+	add_aggregate_row (recset, "log", "comments");
+	add_aggregate_row (recset, "log10", "comments");
+	add_aggregate_row (recset, "lower", "comments");
+	add_aggregate_row (recset, "lpad", "comments");
+	add_aggregate_row (recset, "ltrim", "comments");
+	add_aggregate_row (recset, "make_set", "comments");
+	add_aggregate_row (recset, "master_pos_wait", "comments");
+	add_aggregate_row (recset, "match", "comments");
+	add_aggregate_row (recset, "max", "comments");
+	add_aggregate_row (recset, "md5", "comments");
+	add_aggregate_row (recset, "mid", "comments");
+	add_aggregate_row (recset, "min", "comments");
+	add_aggregate_row (recset, "minute", "comments");
+	add_aggregate_row (recset, "mod", "comments");
+	add_aggregate_row (recset, "month", "comments");
+	add_aggregate_row (recset, "monthname", "comments");
+	add_aggregate_row (recset, "now", "comments");
+	add_aggregate_row (recset, "nullif", "comments");
+	add_aggregate_row (recset, "oct", "comments");
+	add_aggregate_row (recset, "octet_length", "comments");
+	add_aggregate_row (recset, "ord", "comments");
+	add_aggregate_row (recset, "password", "comments");
+	add_aggregate_row (recset, "period_add", "comments");
+	add_aggregate_row (recset, "period_diff", "comments");
+	add_aggregate_row (recset, "pi", "comments");
+	add_aggregate_row (recset, "position", "comments");	
+	add_aggregate_row (recset, "pow", "comments");
+	add_aggregate_row (recset, "power", "comments");
+	add_aggregate_row (recset, "quarter", "comments");
+	add_aggregate_row (recset, "radians", "comments");
+	add_aggregate_row (recset, "rand", "comments");
+	add_aggregate_row (recset, "release_lock", "comments");
+	add_aggregate_row (recset, "repeat", "comments");
+	add_aggregate_row (recset, "replace", "comments");	
+	add_aggregate_row (recset, "reverse", "comments");
+	add_aggregate_row (recset, "right", "comments");
+	add_aggregate_row (recset, "round", "comments");
+	add_aggregate_row (recset, "rpad", "comments");
+	add_aggregate_row (recset, "rtrim", "comments");
+	add_aggregate_row (recset, "second", "comments");
+	add_aggregate_row (recset, "sec_to_time", "comments");
+	add_aggregate_row (recset, "session_user", "comments");
+	add_aggregate_row (recset, "sign", "comments");
+	add_aggregate_row (recset, "sin", "comments");
+	add_aggregate_row (recset, "soundex", "comments");
+	add_aggregate_row (recset, "space", "comments");
+	add_aggregate_row (recset, "sqrt", "comments");
+	add_aggregate_row (recset, "strcmp", "comments");
+	add_aggregate_row (recset, "subdate", "comments");
+	add_aggregate_row (recset, "substring", "comments");
+	add_aggregate_row (recset, "substring_index", "comments");
+	add_aggregate_row (recset, "sysdate", "comments");
+	add_aggregate_row (recset, "system_user", "comments");
+	add_aggregate_row (recset, "tan", "comments");
+	add_aggregate_row (recset, "time_format", "comments");
+	add_aggregate_row (recset, "time_to_sec", "comments");
+	add_aggregate_row (recset, "to_days", "comments");
+	add_aggregate_row (recset, "trim", "comments");
+	add_aggregate_row (recset, "truncate", "comments");
+	add_aggregate_row (recset, "ucase", "comments");
+	add_aggregate_row (recset, "unix_timestamp", "comments");
+	add_aggregate_row (recset, "upper", "comments");
+	add_aggregate_row (recset, "user", "comments");
+	add_aggregate_row (recset, "version", "comments");
+	add_aggregate_row (recset, "week", "comments");
+	add_aggregate_row (recset, "weekday", "comments");
+	add_aggregate_row (recset, "year", "comments");
+	add_aggregate_row (recset, "yearweek", "comments");
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -749,51 +827,119 @@ get_mysql_tables (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
 	GdaMysqlRecordset *recset;
+	GdaDataModelArray *model;
+	gint rows, r;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
-	reclist = process_sql_commands (NULL, cnc, "show tables");
+	reclist = process_sql_commands (NULL, cnc, "show table status");
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_MYSQL_RECORDSET (reclist->data);
+	recset = (GdaMysqlRecordset *) reclist->data;
 	g_list_free (reclist);
+	if (!GDA_IS_MYSQL_RECORDSET (recset))
+		return NULL;
 
-	return GDA_DATA_MODEL (recset);
+	/* add the extra information */
+	model = gda_data_model_array_new (4);
+	rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (recset));
+	for (r = 0; r < rows; r++) {
+		GList *value_list = NULL;
+		gchar *name;
+		gchar *str;
+
+		/* 1st, the name of the table */
+		name = gda_value_stringify (gda_data_model_get_value_at (GDA_DATA_MODEL (recset), 0, r));
+		value_list = g_list_append (value_list, gda_value_new_string (name));
+
+		/* 2nd, the owner */
+		value_list = g_list_append (value_list, gda_value_new_string (""));
+
+		/* 3rd, the comments */
+		str = gda_value_stringify (gda_data_model_get_value_at (GDA_DATA_MODEL (recset), 14, r));
+		value_list = g_list_append (value_list, gda_value_new_string (str));
+		g_free (str);
+
+		/* 4th, the SQL command */
+		str = g_strdup_printf ("SHOW CREATE TABLE %s", name);
+		reclist = process_sql_commands (NULL, cnc, str);
+		g_free (str);
+		if (reclist && GDA_IS_DATA_MODEL (reclist->data)) {
+			str = gda_value_stringify (
+				gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 1, 0));
+			value_list = g_list_append (value_list, gda_value_new_string (str));
+
+			g_free (str);
+			g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
+			g_list_free (reclist);
+		}
+		else
+			value_list = g_list_append (value_list, gda_value_new_string (""));
+
+		gda_data_model_append_row (GDA_DATA_MODEL (model), value_list);
+
+		g_free (name);
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
+	}
+
+	return GDA_DATA_MODEL (model);
 }
 
 static GdaDataModel *
 get_mysql_types (GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModelArray *recset;
+	gint i;
+	struct {
+		const gchar *name;
+		const gchar *owner;
+		const gchar *comments;
+		GdaValueType type;
+	} types[] = {
+		{ "blob", "", "Binary blob", GDA_VALUE_TYPE_BINARY },
+		{ "date", "", "Date", GDA_VALUE_TYPE_DATE },
+		{ "datetime", "", "Date and time", GDA_VALUE_TYPE_TIMESTAMP },
+		{ "decimal", "", "Decimal number", GDA_VALUE_TYPE_DOUBLE },
+		{ "double", "", "Double precision number", GDA_VALUE_TYPE_DOUBLE },
+		{ "enum", "", "Enumeration", GDA_VALUE_TYPE_STRING },
+		{ "float", "", "Single precision number", GDA_VALUE_TYPE_SINGLE },
+		{ "int24", "", "24 bit integer", GDA_VALUE_TYPE_BIGINT },
+		{ "long", "", "Long integer", GDA_VALUE_TYPE_INTEGER },
+		{ "longlong", "", "Extra long integer", GDA_VALUE_TYPE_BIGINT },
+		{ "set", "", "Set", GDA_VALUE_TYPE_STRING },
+		{ "short", "", "Short integer", GDA_VALUE_TYPE_SMALLINT },
+		{ "string", "", "String", GDA_VALUE_TYPE_STRING },
+		{ "time", "", "Time", GDA_VALUE_TYPE_TIME },
+		{ "timestamp", "", "Time stamp", GDA_VALUE_TYPE_TIMESTAMP },
+		{ "tiny", "", "Tiny integer", GDA_VALUE_TYPE_SMALLINT },
+		{ "year", "", "Year", GDA_VALUE_TYPE_INTEGER }
+	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	/* create the recordset */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (1);
-	//gda_server_recordset_model_set_field_defined_size (recset, 0, 32);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (4);
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
-	//gda_server_recordset_model_set_field_scale (recset, 0, 0);
-	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
 
 	/* fill the recordset */
-	add_string_row (recset, "blob");
-	add_string_row (recset, "date");
-	add_string_row (recset, "datetime");
-	add_string_row (recset, "decimal");
-	add_string_row (recset, "double");
-	add_string_row (recset, "enum");
-	add_string_row (recset, "float");
-	add_string_row (recset, "int24");
-	add_string_row (recset, "long");
-	add_string_row (recset, "longlong");
-	add_string_row (recset, "set");
-	add_string_row (recset, "short");
-	add_string_row (recset, "string");
-	add_string_row (recset, "time");
-	add_string_row (recset, "timestamp");
-	add_string_row (recset, "tiny");
-	add_string_row (recset, "year");
+	for (i = 0; i < sizeof (types) / sizeof (types[0]); i++) {
+		GList *value_list = NULL;
+
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
+		value_list = g_list_append (value_list, gda_value_new_type (types[i].type));
+
+		gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
+	}
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -852,6 +998,9 @@ field_row_to_value_list (MYSQL_ROW mysql_row)
 	/* references */
 	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[5]));
 
+	/* default value */
+	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[4]));
+
 	return value_list;
 }
 
@@ -869,7 +1018,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	struct {
 		const gchar *name;
 		GdaValueType type;
-	} fields_desc[8] = {
+	} fields_desc[] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Size")		, GDA_VALUE_TYPE_INTEGER },
@@ -877,7 +1026,8 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 		{ N_("Not null?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("References")	, GDA_VALUE_TYPE_STRING  }
+		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -894,7 +1044,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	if (!par) {
 		gda_connection_add_error_string (
 			cnc,
-			_("You need to specify the name of the table"));
+			_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
@@ -902,7 +1052,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	if (!table_name) {
 		gda_connection_add_error_string (
 			cnc,
-			_("You need to specify the name of the table"));
+			_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
@@ -919,7 +1069,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	rows = mysql_num_rows (mysql_res);
 
 	/* fill in the recordset to be returned */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (8);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (9);
 	for (r = 0; r < sizeof (fields_desc) / sizeof (fields_desc[0]); r++) {
 		//gint defined_size =  (fields_desc[r].type == GDA_VALUE_TYPE_STRING) ? 64 : 
 		//	(fields_desc[r].type == GDA_VALUE_TYPE_INTEGER) ? sizeof (gint) : 1;
