@@ -25,6 +25,7 @@
 #include <config.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gda-mysql.h"
 #include "gda-mysql-recordset.h"
 
@@ -91,30 +92,30 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
 		switch (mysql_fields[i].type) {
 		case FIELD_TYPE_DECIMAL :
 		case FIELD_TYPE_DOUBLE :
-			gda_value_set_double (field, atof (thevalue));
+			gda_value_set_double (field, thevalue ? atof (thevalue) : 0.0);
 			break;
 		case FIELD_TYPE_FLOAT :
-			gda_value_set_single (field, atof (thevalue));
+			gda_value_set_single (field, thevalue ? atof (thevalue) : 0.0);
 			break;
 		case FIELD_TYPE_LONG :
 		case FIELD_TYPE_YEAR :
-			gda_value_set_integer (field, atol (thevalue));
+			gda_value_set_integer (field, thevalue ? atol (thevalue) : 0);
 			break;
 		case FIELD_TYPE_LONGLONG :
 		case FIELD_TYPE_INT24 :
-			gda_value_set_bigint (field, atoll (thevalue));
+			gda_value_set_bigint (field, thevalue ? atoll (thevalue) : 0);
 			break;
 		case FIELD_TYPE_SHORT :
-			gda_value_set_smallint (field, atoi (thevalue));
+			gda_value_set_smallint (field, thevalue ? atoi (thevalue) : 0);
 			break;
 		case FIELD_TYPE_TINY :
-			gda_value_set_tinyint (field, atoi (thevalue));
+			gda_value_set_tinyint (field, thevalue ? atoi (thevalue) : 0);
 			break;
 		case FIELD_TYPE_TINY_BLOB :
 		case FIELD_TYPE_MEDIUM_BLOB :
 		case FIELD_TYPE_LONG_BLOB :
 		case FIELD_TYPE_BLOB :
-			gda_value_set_binary (field, thevalue, lengths[i]);
+			gda_value_set_binary (field, thevalue, mysql_fields[i].max_length);
 			break;
 		case FIELD_TYPE_VAR_STRING :
 		case FIELD_TYPE_STRING :
@@ -165,7 +166,7 @@ gda_mysql_recordset_describe_column (GdaDataModel *model, gint col)
 {
 	gint field_count;
 	GdaFieldAttributes *attrs;
-	MYSQL_FIELD *mysql_fields;
+	MYSQL_FIELD *mysql_field;
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
@@ -182,15 +183,22 @@ gda_mysql_recordset_describe_column (GdaDataModel *model, gint col)
 
 	attrs = gda_field_attributes_new ();
 
-	mysql_fields = mysql_fetch_field (recset->mysql_res);
-	if (!mysql_fields)
+	mysql_field = mysql_fetch_field_direct (recset->mysql_res, col);
+	if (!mysql_field) {
+		gda_field_attributes_free (attrs);
 		return NULL;
+	}
 
-	if (mysql_fields[col].name)
-		gda_field_attributes_set_name (attrs, mysql_fields[col].name);
-	gda_field_attributes_set_defined_size (attrs, mysql_fields[col].max_length);
-	gda_field_attributes_set_scale (attrs, mysql_fields[col].decimals);
-	gda_field_attributes_set_gdatype (attrs, gda_mysql_type_to_gda (mysql_fields[col].type));
+	if (mysql_field->name)
+		gda_field_attributes_set_name (attrs, mysql_field->name);
+	gda_field_attributes_set_defined_size (attrs, mysql_field->length);
+	gda_field_attributes_set_table (attrs, mysql_field->table);
+	gda_field_attributes_set_scale (attrs, mysql_field->decimals);
+	gda_field_attributes_set_gdatype (attrs, gda_mysql_type_to_gda (mysql_field->type));
+	gda_field_attributes_set_allow_null (attrs, !IS_NOT_NULL (mysql_field->flags));
+	gda_field_attributes_set_primary_key (attrs, IS_PRI_KEY (mysql_field->flags));
+	gda_field_attributes_set_unique_key (attrs, mysql_field->flags & UNIQUE_KEY_FLAG);
+	gda_field_attributes_set_auto_increment (attrs, mysql_field->flags & AUTO_INCREMENT_FLAG);
 
 	return attrs;
 }
@@ -433,6 +441,7 @@ GdaMysqlRecordset *
 gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res)
 {
 	GdaMysqlRecordset *recset;
+	MYSQL_FIELD *mysql_fields;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (mysql_res != NULL, NULL);
@@ -441,5 +450,15 @@ gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res)
 	recset->cnc = cnc;
 	recset->mysql_res = mysql_res;
 
+	mysql_fields = mysql_fetch_fields (recset->mysql_res);
+	if (mysql_fields != NULL) {
+		gint i;
+
+		for (i = 0; i < mysql_num_fields (recset->mysql_res); i++) {
+			gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
+							 i, mysql_fields[i].name);
+		}
+	}
+
 	return recset;
 }
