@@ -25,7 +25,6 @@
 #  define __gda_postgres_provider_h__
 
 #include <libgda/gda-server-provider.h>
-#include <libgda/gda-server-recordset-model.h>
 #include <libpq-fe.h>
 
 #define GDA_TYPE_POSTGRES_PROVIDER            (gda_postgres_provider_get_type())
@@ -53,7 +52,9 @@ struct _GdaPostgresProviderClass {
 typedef struct {
 	gchar *name;
 	Oid oid;
-	GdaType type;
+	GdaValueType type;
+	gchar *comments;
+	gchar *owner;
 } GdaPostgresTypeOid;
 
 typedef struct {
@@ -61,12 +62,14 @@ typedef struct {
 	gint ntypes;
 	GdaPostgresTypeOid *type_data;
 	GHashTable *h_table;
+	gchar *version;
 } GdaPostgresConnectionData;
 
 
 G_BEGIN_DECLS
 
-GType gda_postgres_provider_get_type (void);
+GType              gda_postgres_provider_get_type (void);
+GdaServerProvider *gda_postgres_provider_new (void);
 
 G_END_DECLS
 
