@@ -21,9 +21,11 @@
  */
 
 #include <config.h>
+#include <string.h>
 #include <libgda/gda-intl.h>
 #include "gda-test.h"
 #include "postgres-test.h"
+#include "freetds-test.h"
 
 /* Shows the object schemas */
 static void
@@ -70,6 +72,7 @@ open_connection (GdaClient *client,
 	GdaConnection *cnc;
 	gboolean res;
 	GdaDataSourceInfo *dsn_info;
+	GdaTransaction *xaction;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
@@ -113,6 +116,8 @@ open_connection (GdaClient *client,
 		 _("Supported") : _("Not supported"));
 
 	/* show connection schemas */
+	g_print (_("\n\tProvider reports version: %s\n\n"),
+			gda_connection_get_server_version (cnc));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_DATABASES, _("Databases"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_PROCEDURES, _("Stored procedures"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_TABLES, _("Connection Tables"));
@@ -125,26 +130,32 @@ open_connection (GdaClient *client,
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_TRIGGERS, _("Triggers"));
 
 	/* test transactions */
+	xaction = gda_transaction_new (NULL);
+
 	g_print (_("\tStarting transaction..."));
-	res = gda_connection_begin_transaction (cnc, NULL);
+	res = gda_connection_begin_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tFinishing transaction..."));
-	res = gda_connection_commit_transaction (cnc, NULL);
+	res = gda_connection_commit_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tStarting transaction..."));
-	res = gda_connection_begin_transaction (cnc, NULL);
+	res = gda_connection_begin_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tRolling back transaction..."));
-	res = gda_connection_rollback_transaction (cnc, NULL);
+	res = gda_connection_rollback_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
+	g_object_unref (G_OBJECT (xaction));
+
 	dsn_info = gda_config_find_data_source (name);
 	/* Postgres own tests */
 	if (!strcmp (dsn_info->provider, POSTGRES_PROVIDER_NAME))
 		do_postgres_test (cnc);
+	if (!strcmp (dsn_info->provider, FREETDS_PROVIDER_NAME))
+		do_freetds_test (cnc);
 	gda_config_free_data_source_info (dsn_info);
 
 	/* close the connection */
